"""
        Yasuharu Okamoto, 2018

"""
import pystan
from pystan import StanModel
import pickle
import random
import math
import matplotlib.pyplot as plt

def icc(t, a, b, c):
    return 1.0 / (1.0 + math.exp(-1.7 * a * (t - b - c)))

def CatResponse(t, a, b, c, K):
    res = 1
    v = random.random()
    while True:
        if v > icc(t, a, b, c[res - 1]):
            break
        res += 1
        if res >= K:
            break
    return res

def calcCoeff(x, y):
    sumx = 0.0
    sumy = 0.0
    n = len(x)
    for i in range(n):
        sumx += x[i]
        sumy += y[i]
    meanx = sumx / n
    meany = sumy / n
    sumxx = 0.0
    sumyy = 0.0
    sumxy = 0.0
    for i in range(n):
        sumxx += (x[i] - meanx) ** 2
        sumyy += (y[i] - meany) ** 2
        sumxy += (x[i] - meanx) * (y[i] - meany)
    Cxy = sumxy / n
    Sx = (sumxx / n) ** 0.5
    Sy = (sumyy / n) ** 0.5
    r = Cxy / (Sx * Sy)
    a = r * Sy / Sx
    b = -Cxy * meanx / (Sx ** 2) + meany
        
    return r, a, b

Nitems = 5                         #    Number of items
X = [1, 1, 1, 1, 1]                #    Responses to be set in simulation

a = [1.0, 1.0, 1.0, 1.0, 1.0]      #    Discrimination parameters
b = [-0.5, -0.3, 0.0, 0.3, 0.5]    #    Difficulty parameters
K = 2                              #    Number of response categories
C = [0.0]                          #    Category boundaries
#   K = 7
#   C = [-2.0, -1.0, -0.25, 0.25, 1.0, 2.0]

Data = {'K': K, 'Nitm': Nitems,  'C': C, 'a':a, 'b': b, 'X': X}

sm = StanModel(file = 'EstTheta.stan')
with open('model.pkl', 'wb') as f:
    pickle.dump(sm, f)

L_theta1 = []
L_thetaTrue = []
n_step = 1000
for step in range(n_step):
    if (step % 5 == 0):
        print('\nstep = {0}/{1}'.format(step, n_step))
    t = random.gauss(0.0, 1.0)
    L_thetaTrue.append(t)
    for i in range(Nitems):
        X[i] = CatResponse(t, a[i], b[i], C, K)
    sm = pickle.load(open('model.pkl', 'rb'))
    fit = sm.sampling(data = Data,  n_jobs = 1)
    theta = fit['theta']
    theta.sort()
    L_theta1.append(theta[int(len(theta) / 2)])
 
r, coeff_a, coeff_b = calcCoeff(L_thetaTrue, L_theta1)

min_t = L_thetaTrue[0]
max_t = min_t
n = len(L_thetaTrue)
for i in range(n):
    if min_t > L_thetaTrue[i]:
        min_t = L_thetaTrue[i]
    if max_t < L_thetaTrue[i]:
        max_t = L_thetaTrue[i]

regression_x = [min_t, max_t]
regression_y = [coeff_a * min_t + coeff_b, coeff_a * max_t + coeff_b]
plt.plot(regression_x, regression_y, 'b-')

plt.title("Predictive power = {0:>.3}\nr = {1:>.3}".format(r**2, r))
plt.xlabel("True theta")    
plt.ylabel("Est. theta")     
plt.plot(L_thetaTrue, L_theta1, 'bo')    
plt.show()  

